COLUMNS_CONSIDERED = [
    'Motor Speed (RPM)',
    'Engine Speed (RPM)',
    'Throttle (%)',
    'Intake Temperature (C)',
    'Engine Coolant Temperature 1 (C)',
    'Engine Coolant Temperature 2 (C)',
    'Barometric Pressure (kpa)',
    'Fuel Trim',
    'Fuel Consumption (g/min)',
    'Fuel Consumed (g)',
    'Bus Voltage (V)',
    'Battery Current (A)',
    'Power Generated (W)',
    'Inverter Temperature (C)',
    'Target Fuel Pressure (bar)',
    'Fuel Pressure (bar)',
    'Fuel Pump Speed (RPM)',
    'Cooling Pump Speed (RPM)',
]
